//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : BigRational Driver
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  School of Information and Computer Science
//                  University of California at Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This program implements a driver for the BIgRational class. Most of
// its menu selections allow the user to call the individual methods
// declared in this class (all accessors) on an object that it
// constructs. Other menu items perform combinations of these methods.
// In this way, programmers can learn about this class by experimenting
// with its methods.
//
// Known Bugs     : None
//
// Future Plans   : None
//
// Program History:
//   9/11/01: R. Pattis - Operational in Java
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import java.math.BigInteger;
import edu.uci.ics.pattis.ics22.BigRational;
import edu.uci.ics.pattis.introlib.Prompt;
import edu.uci.ics.pattis.introlib.Timer;


public class BigRationalDriver {


  //Present menu and get user's choice
  
  private static char menuPrompt ()
  {
    System.out.println("\n\nAccessors      General");
    System.out.println("  + - add        h - harmonic sum: 1/1 + 1/2 + ... + 1/n");
    System.out.println("  - - subtract   e - sum: 1 + 1/1! + 1/2! + ... 1/n!");
    System.out.println("  * - multiply   q - quit");
    System.out.println("  / - divide");
    System.out.println("  a - abs");
    System.out.println("  n - negate");
    System.out.println("  r - reciprocal");
    System.out.println("  ? - compareTo/equals");
    System.out.println("  c - toDecimalString");
    
    return Prompt.forChar("\nEnter Command","+-*/anr?cheq");
  }
  
  
  
  ////////////////
  //Driver Program
  ////////////////
  
 
	public static void main(String[] args)
	{
	  BigRational Left, Right;
	  
	  while (true)
	    try {
        switch(menuPrompt()) {
          case '+' : Left  = BigRational.prompt("Enter Left  Argument");
                     Right = BigRational.prompt("Enter Right Argument");
                     System.out.println(Left + " + " + Right + " = " + (Left.add(Right)));
                     break;

          case '-' : Left  = BigRational.prompt("Enter Left  Argument");
                     Right = BigRational.prompt("Enter Right Argument");
                     System.out.println(Left + " - " + Right + " = " + (Left.subtract(Right)));
                     break;

          case '*' : Left  = BigRational.prompt("Enter Left  Argument");
                     Right = BigRational.prompt("Enter Right Argument");
                     System.out.println(Left + " * " + Right + " = " + (Left.multiply(Right)));
                     break;

          case '/' : Left  = BigRational.prompt("Enter Left  Argument");
                     Right = BigRational.prompt("Enter Right Argument");
                     System.out.println(Left + " / " + Right + " = " + (Left.divide(Right)));
                     break;

          case 'a' : Right  = BigRational.prompt("Enter Argument");
                     System.out.println("absolute value of " + Right + " = " + Right.abs());
                     break;
 
          case 'n' : Right  = BigRational.prompt("Enter Argument");
                     System.out.println("negation of " + Right + " = " + Right.negate());
                     break;
 
          case 'r' : Right  = BigRational.prompt("Enter Argument");
                     System.out.println("reciprocal of " + Right + " = " + Right.reciprocal());
                     break;

          case '?' : Left  = BigRational.prompt("Enter Left  Argument");
                     Right = BigRational.prompt("Enter Right Argument");
                     System.out.println(Left + ".equals(" + Right + ") = " + Left.equals(Right));
                     System.out.println(Left + ".compareTo(" + Right + ") = " + Left.compareTo(Right));
                     break;

          case 'c' : {
                       Right  = BigRational.prompt("Enter Argument");
                       int decs = Prompt.forInt("Enter decimal places");
                       System.out.println("Decimal of " + Right + " = " + Right.toDecimalString(decs));
                       break;
                     }

          case 'h' : {
        	  			     int bound = Prompt.forInt("Enter n");
         			         Timer t = new Timer(); 
         			         t.start();
         			         
        		   			   BigRational sum = new BigRational();
        				  	   for (int i=1; i<=bound; i++)
        					       sum = sum.add(new BigRational(BigInteger.ONE,new BigInteger(""+i)));
         					     t.stop();
        					     System.out.println("Elapsed time = " + t.getElapsed() + " seconds");
        					     
        					     System.out.println("1/1 + 1/2 + ... + 1/"+bound+" = " + sum);
                       int decs  = Prompt.forInt("Enter decimal places");

        					     System.out.println("1/1 + 1/2 + ... + 1/"+bound+" = " + sum.toDecimalString(decs));
                       break;
                     }

          case 'e' : {
        	  			     int bound = Prompt.forInt("Enter n");
        			         Timer t = new Timer();
        			         t.start();
        			         
        		   			   BigRational sum = BigRational.ONE.add(new BigRational(BigInteger.ONE,new BigInteger(""+bound)));
        				  	   for (int i=bound-1; i>0; i--)
        					       sum = BigRational.ONE.add(sum.multiply(new BigRational(BigInteger.ONE,new BigInteger(""+i))));
        					     t.stop();
        					     System.out.println("Elapsed time = " + t.getElapsed() + " seconds");
        					      
        					     System.out.println("1 + 1/1! + 1/2! + ... + 1/"+bound+"! = " + sum);
                       int decs  = Prompt.forInt("Enter decimal places");

        					     System.out.println("1 + 1/1! + 1/2! + ... + 1/"+bound+"! = " + sum.toDecimalString(decs));
        					     System.out.println("Math.E = " + Math.E);
                       break;
                     }

         case 'q' : return;

         default  : System.out.println("Unknown command");
                    break;

         }

		}catch (Exception e) {
	    e.printStackTrace();
	    System.out.println("Application class terminating");
	    System.exit(0);  
	  }
  }
  
  
}
