//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : SimplePriorityQueue Driver
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  School of Information and Computer Science
//                  University of California at Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
//   This program implements a driver for the SimplePriorityQueue class.
// Most of its menu selections allow the user to call the individual
// methods declared in this class (both mutators and accessors) on an object
// that it constructs. Other menu items perform combinations of these methods.
// In this way, programmers can learn about this class by experimenting with
// its methods.
//
// Known Bugs     : None
//
// Future Plans   : None
//
// Program History:
//   9/24/01: R. Pattis - Operational in Java
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import edu.uci.ics.pattis.introlib.Prompt;

//Some errors below relate to these classes not existing

//import edu.uci.yourname.ics22.SimplePriorityQueue;
//import edu.uci.yourname.ics22.StringByLength;
//import edu.uci.yourname.ics22.StringByLengthReverse;
//import edu.uci.yourname.ics22.StringByValue;
//import edu.uci.yourname.ics22.StringByValueIgnoreCase;
//import edu.uci.yourname.ics22.IntegerIncreasing;


import java.util.Comparator;


public class SPQDriver {


  //Present menu and get user's choice
  
  private static char menuPrompt ()
  {
    System.out.println("\n\nAccessors          Mutator         General");
    System.out.println("  ? - isEmpty        m - makeEmpty   f - flush: dequeue all");
    System.out.println("  s - getSize        e - enqueue     i - integer tes");
    System.out.println("  p - peek           d - dequeue     q - quit");
    System.out.println("  t - toString");
    
    return Prompt.forChar("\nEnter Command","?sptmedrfiq");
  }
  
  
  
  ////////////////
  //Driver Program
  ////////////////
  
	public static void main(String[] args)
	{
    //Construct object to test; let the user specify what Comparator
    //  object to construct to order the priority queue
    
	  SimplePriorityQueue pq;
	  char howPrioritize = Prompt.forChar("Enter how to prioritize queue\n  v (by value)\n  i (value ignore case)\n  l (by length)\n  r (by reverse length)\n\nChoice","virl");
	  if (howPrioritize == 'v')
	    pq = new SimplePriorityQueue(new StringByValue());
	  else if (howPrioritize == 'i')
	    pq = new SimplePriorityQueue(new StringByValueIgnoreCase());
	  else if (howPrioritize == 'l')
	    pq = new SimplePriorityQueue(new StringByLength());
	  else
	    pq = new SimplePriorityQueue(new StringByLengthReverse());
	  
	  
	  //Test priority queue object using any method
	  for (;;) 
	    try {
  	    char selection = menuPrompt();
  	    
        if (selection == '?')
          System.out.println("  isEmpty = " + pq.isEmpty());
          
        else if (selection == 's')
          System.out.println("  size = " + pq.getSize());
       
        else if (selection == 'p')
          System.out.println("  peek = " + pq.peek());

        else if (selection == 't')
          System.out.println("  toString = " + pq);

        else if (selection == 'm')
          pq.makeEmpty();
          
        else if (selection == 'e')
          pq.enqueue(Prompt.forString("  Enter String to enqueue"));
          
        else if (selection == 'd')
          System.out.println("  dequeue = " + pq.dequeue());
          
        else if (selection == 'f')
          for(;!pq.isEmpty();)
  	        System.out.println("  dequeue = " + pq.dequeue());
  	    
        else if (selection == 'i') {
          SimplePriorityQueue temp = new SimplePriorityQueue(new IntegerIncreasing());
          int numberToQueue = Prompt.forInt("Enter # of integers for queue");
          System.out.print("Enqueue order = ");
          for (int i=1; i<=numberToQueue; i++) {
            int toEnqueue = (int) (1000*Math.random());
            System.out.print(toEnqueue +" ");
            temp.enqueue(new Integer(toEnqueue));
          }
          
          System.out.print("\nDequeue order = ");
          for(;!temp.isEmpty();)
  	        System.out.print(temp.dequeue()+" ");
  	    }
  	    
        else if (selection == 'q')
          break;
        
        else
          System.out.println("\""+selection+"\" is unknown command");
 
      }catch(Exception e) {
        System.out.println("  Exception Caught/Handled: "+e.getMessage());
      }
  }
}
