///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
//
// Class          : vendingMachine.Model
//
// Author         : Richard E. Pattis
//                   Computer Science Department
//                   School of Information and Computer Science
//                   University of California at Irvine
//                   Irvine, CA 92617-3435
//                   e-mail: pattis@ics.uci.edu
//
// Maintainer       : Author
//
//
// Description:
//
// The Model for the VendingMachine package implements the guts of the
// vending machine: it responds to presses of buttons created by the
// Conroller (deposit, cancel, buy), and tells the View when it needs
// to update its display (calling the update in view, which calls the
// accessor methods in this classes)
//
// Note that "no access modifier" means that the method is package
// friendly: this means the member is public to all other classes in
// the calculator package, but private elsewhere.
//
// Future Plans   : More Comments
//                  Increase price as stock goes down
//                  Decrease price if being outsold by competition
//                  Allow option to purchase even if full change cannot 
//                    be returned (purchaser pays a premium to quench thirst)
//                  Allow user to enter 2 x money and gamble: 1/2 time
//                    all money returned with product; 1/2 time no money and
//                    no product returned
//
// Program History:
//    9/20/01: R. Pattis - Operational for 15-100
//    2/10/02: R. Pattis - Fixed Bug in change-making method
//   10/10/07: R. Pattis - Added main (Text Driver) and Updateable interface
//                           for view in Text Driver.
//
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////


package vendingMachine;


import java.lang.Math;
import edu.uci.ics.pattis.introlib.Prompt;

public class Model {

  //Define constructor; remember is should prompt user for
	//  initial amounts and prices
   
	//Refer to the view (used to call update after each button press)
	//The View implements Updateable

	public void addView(Updateable v)
	{view = v;}
	
  //Define required methods: mutators and accessors
  
	
	//Represent "interesting" state of vending machine
	public String toString() {
	  return "Vending Machine State: \n" +
	    "  Coke     Left      = " + cokeLeft     + "\n" +
	    "  Pepsi    Left      = " + pepsiLeft    + "\n" +
	    "  Quarters Left      = " + quartersLeft + "\n" +
	    "  Dimes    Left      = " + dimesLeft    + "\n" +
	    "  Nickels  Left      = " + nickelsLeft  + "\n";
	    //Display any other instance variables that you declare too, and there should be others
  }

  //Define (private) helper methods

  //Define fields (all instance variables)
	     
	private Updateable view;         // Model must tell View when to update itself, so must access view
	 
	private int    cokeLeft;
	private int    pepsiLeft;
	 
	private int    quartersLeft, dimesLeft, nickelsLeft;
	 
	//I defined about 10 more fields, besides the ones above



	
	
	//////////////////////////////////////////////////
	//
	//Text Driver for Model (not using View or Controller
	//
	//////////////////////////////////////////////////
 
	private static char menuPrompt () {
    System.out.println("\n\nText Driver for Vending Machine Model");;
    System.out.println("Mutators          General");
    System.out.println("  d - deposit       s - show all state");
    System.out.println("  b - buy           q - quit");
    System.out.println("  c - cancel");
     
    return Prompt.forChar("\nEnter Command","dbcsq");
	}
	
	public static void main(String[] args) {
		Model test = new Model();
		test.addView( new Updateable() {
			public void update()
			{System.out.println("update called in View");}
		});
		
		for (;;) {
			switch (menuPrompt()) {
			  case 'd' : {
			  	int amount;
			  	for(;;) {
				  	amount = Prompt.forInt("  Enter amount to deposit (5, 10, or 25)");
			  		if (amount == 5 || amount == 10 | amount == 25)
			  		  break;
			  		System.out.println("    Error, please reenter correct amount");
			  	}
			  	test.deposit(amount);
			  	break;
			  }
			  
			  case 'b' : {
	  	    String product = Prompt.forString("  Enter amount to deposit", new String[]{"Coke","Pepsi"});
	  	    test.buy(product);
	  	    break;
		    }

	      case 'c' : {
          test.cancel();
          break;
        }

        case 's' : {
          System.out.println("  getMessage   () = " + test.getMessage   ());
          System.out.println("  getDeposited () = " + test.getDeposited ());
          System.out.println("  getCokeLeft  () = " + test.getCokeLeft  ());
          System.out.println("  getPepsiLeft () = " + test.getPepsiLeft ());
          System.out.println("  getCokePrice () = " + test.getCokePrice ());
          System.out.println("  getPepsiPrice() = " + test.getPepsiPrice());
          System.out.println("  toString        = " + test.toString());
          break;
        }

	      case 'q' : break; 
	    }
	  }
	}
}
